@@ -31,6 +31,7 @@ class xlcv(EnchantBase):
         """
         # 虽然只绑定cv2，但其他相关的几个库的方法上，最好也不要重名
         names = cls.check_enchant_names([np.ndarray, PIL.Image, PIL.Image.Image])
+        names -= {'concat'}
         cls._enchant(cv2, names, EnchantCvt.staticmethod2modulefunc)
 
     @staticmethod
@@ -555,6 +556,100 @@ class xlcv(EnchantBase):
             dst = xlcv.warp(dst, warp_mat, (w, h))
         return dst
 
+    @staticmethod
+    def trim(im, *, border=0, color=None):
+        """ 如果想用cv2实现，可以参考： https://stackoverflow.com/questions/49907382/how-to-remove-whitespace-from-an-image-in-opencv
+        目前为了偷懒节省代码量，就直接调用pil的版本了
+        """
+        from pyxllib.cv.xlpillib import xlpil
+        im = xlcv.to_pil_image(im)
+        im = xlpil.trim(im, border=border, color=color)
+        return xlpil.to_cv2_image(im)
+
+    @staticmethod
+    def keep_subtitles(im, judge_func=None, trim_color=(255, 255, 255)):
+        """ 保留（白色）字幕，去除背景，并会裁剪图片缩小尺寸
+
+        是比较业务级的一个功能，主要是这段代码挺有学习价值的，有其他变形需求，
+        可以修改judge_func，也可以另外写函数，这个仅供参考
+        """
+
+        def fore_pixel(rgb):
+            """ 把图片变成白底黑字
+
+            判断像素是不是前景，是返回0，不是返回255
+            """
+            if sum(rgb > 170) == 3 and rgb.max() - rgb.min() < 30:
+                return [0, 0, 0]
+            else:
+                return [255, 255, 255]
+
+        if judge_func is None:
+            judge_func = fore_pixel
+
+        im2 = np.apply_along_axis(judge_func, 2, im).astype('uint8')
+        if trim_color:
+            im2 = xlcv.trim(im2, color=trim_color)
+        return im2
+
+    @classmethod
+    def concat(cls, images, *, pad=5, pad_color=None):
+        """ 拼接输入的多张图为一张图，请自行确保尺寸匹配
+
+        :param images: 一维或二维list数组存储的np.array矩阵
+            一维的时候，默认按行拼接，变成 n*1 的图片。如果要显式按列拼接，请再套一层list，输入 [1, n] 的list
+        :param pad: 图片之间的间隔，也可以输入 [5, 10] 表示左右间隔5像素，上下间隔10像素
+        :param pad_color: 填充颜色，默认白色
+
+        # TODO 下标编号
+        """
+        if pad_color is None:
+            pad_color = 255
+
+        if isinstance(pad, int):
+            pad = [pad, pad]
+
+        def hstack(imgs):
+            """ 水平拼接图片 """
+            patches = []
+
+            max_length = max([im.shape[0] for im in imgs])
+            max_channel = max([xlcv.n_channels(im) for im in imgs])
+            board = np.ones([max_length, pad[1]], dtype='uint8') * pad_color
+            if max_channel == 3:
+                board = xlcv.cvt_channel(board, 1)
+
+            for im in imgs:
+                h = im.shape[0]
+                if h != max_length:
+                    im = xlcv.pad(im, [0, max_length - h, 0, 0])
+                if max_channel == 3:
+                    im = xlcv.cvt_channel(im, 1)
+                patches += [board, im]
+            return np.hstack(patches[1:])
+
+        def vstack(imgs):
+            patches = []
+
+            max_length = max([im.shape[1] for im in imgs])
+            max_channel = max([xlcv.n_channels(im) for im in imgs])
+            board = np.ones([pad[0], max_length], dtype='uint8') * pad_color
+            if max_channel == 3:
+                board = xlcv.cvt_channel(board, 1)
+
+            for im in imgs:
+                w = im.shape[1]
+                if w != max_length:
+                    im = xlcv.pad(im, [0, 0, 0, max_length - w])
+                if max_channel == 3:
+                    im = xlcv.cvt_channel(im, 1)
+                patches += [board, im]
+            return np.vstack(patches[1:])
+
+        if isinstance(images[0], list):
+            images = [hstack(imgs) for imgs in images]
+        return vstack(images)
+
 
 class CvImg(np.ndarray):
     def __new__(cls, input_array, info=None):
